
local C = require("classic")
local M = require("menu")


-- Check if 'multiwand' module is loaded and if it is,
-- use better matching sprites, otherwise simpler

if (package.loaded.multiwand) then   
   g_wand_tele = kts.Graphic("gfx/tele.bmp",255,255,0,-3,-6) --Pickup
   g_wand_tele_north = kts.Graphic("gfx/tele_north.bmp",255,255,0, -6,-5) --Held N
   g_wand_tele_east  = kts.Graphic("gfx/tele_east.bmp" ,255,255,0,  0,-6) --Held E
   g_wand_tele_south = kts.Graphic("gfx/tele_south.bmp",255,255,0, -7, 0) --Held S
   g_wand_tele_west  = kts.Graphic("gfx/tele_west.bmp" ,255,255,0, -5,-7) --Held W
else
   g_wand_tele = kts.Graphic("gfx/tele2.bmp",255,255,0,-3,-6) --Pickup
   g_wand_tele_north = kts.Graphic("gfx/tele2_north.bmp",255,255,0, -6,-5) --Held N
   g_wand_tele_east  = kts.Graphic("gfx/tele2_east.bmp" ,255,255,0,  0,-6) --Held E
   g_wand_tele_south = kts.Graphic("gfx/tele2_south.bmp",255,255,0, -7, 0) --Held S
   g_wand_tele_west  = kts.Graphic("gfx/tele2_west.bmp" ,255,255,0, -5,-7) --Held W
end

-- Overlays for graphics
o_wand_tele = kts.Overlay{ g_wand_tele_north, g_wand_tele_east, g_wand_tele_south, g_wand_tele_west }


-- A hack for figuring out the dimensions of the dungeon
local thelayout = nil
local mapwidth = 0
local mapheight = 0

function getlayout()
   if thelayout == nil then
      thelayout = C.Dsetup.layout.func()
      mapwidth  = (thelayout.width  * 12) + thelayout.width  + 1
      mapheight = (thelayout.height * 12) + thelayout.height + 1
   end
end

-- Random XY teleport function
function teleport_random_xy(object)
   if object == nil then 
      return
   end
   local xmin = 1
   local ymin = 1
   local xmax = mapwidth
   local ymax = mapheight
     -- print("xmin = " .. xmin .. ", ymin = " .. ymin .. ".")
     -- print("xmax = " .. xmax .. ", ymax = " .. ymax .. ".")
   local new_pos = { x, y }
   new_pos.x = kts.RandomRange(xmin, xmax)
   new_pos.y = kts.RandomRange(ymin, ymax)
     -- print("x = " .. new_pos.x .. ", y = " .. new_pos.y .. ".")
   C.snd_teleport()
   kts.TeleportTo(object, new_pos)
   C.snd_teleport()
end

-- List of actors that have picked the wand up
wand_users = {}

function wanduser_add(user)
   if kts.IsAlive(user) then
      -- Check if the actor is already in the list
      if (wand_users ~= nil) then
         for i=1, #wand_users do
            if user == wand_users[i] then 
               return
            end
         end
      end
      -- Not in the list, so add it
      table.insert(wand_users, user)
   end
end

-- telewand zapping function
function telewandzap()
   getlayout()
   if cxt.tile == C.t_crystal_ball then
      -- Teleport everyone who has touched a telewand
      for i=1, #wand_users do
         if kts.IsAlive(wand_users[i]) then 
            teleport_random_xy(wand_users[i])
         end
      end
      C.wandzap()
   else
      if cxt.victim ~= nil then
         C.wandzap()
         teleport_random_xy(cxt.victim)
      end
   end
end


-- The item definition
i_wand_of_teleport = kts.ItemType(
  C.basic_wand & {
    melee_damage = 0,
    melee_stun_time = C.rng_time_range(2, 3),
    melee_tile_damage = 0,
    graphic = g_wand_tele,
    overlay = o_wand_tele,
    melee_action = function()
       telewandzap()
    end,
    on_pick_up = function()
       wanduser_add(cxt.actor)
    end,
    allow_strenght = false
  }
)

-- Update the "all_wands" table, so that this wand counts as one.
table.insert(C.all_wands, i_wand_of_teleport)
